from typing import List, Type
from nb_cli import handlers as _handlers
from nb_cli import config as _config
from nb_cli.config import Driver, Plugin, Adapter


class _nb_cli:
    handlers = _handlers
    config = _config


nb_cli: Type[_nb_cli]

RawInfo = dict[
    {
        "module_name": str,
        "project_link": str,
        "name": str,
        "desc": str,
        "author": str,
        "homepage": str,
        "tags": List[dict[{"label": str, "color": str}]],
        "is_official": bool
    }
]


class _meta:
    drivers: List[Driver]
    adapters: List[Adapter]
    plugins: List[Plugin]
    raw_drivers: List[RawInfo]
    raw_adapters: List[RawInfo]
    raw_plugins: List[RawInfo]


meta: Type[_meta]


from functools import lru_cache
from typing import Any, Dict, List, Literal


@lru_cache
def load_module_data_raw(
    module_name: Literal["adapters", "plugins", "drivers"]
) -> List[Dict[str, Any]]:
    """Get raw module data."""
    from concurrent.futures import ThreadPoolExecutor, as_completed
    import httpx
    exceptions: List[Exception] = []
    urls = [
        f"https://v2.nonebot.dev/{module_name}.json",
        f"https://raw.fastgit.org/nonebot/nonebot2/master/website/static/{module_name}.json",
        f"https://cdn.jsdelivr.net/gh/nonebot/nonebot2/website/static/{module_name}.json",
    ]
    with ThreadPoolExecutor(max_workers=5) as executor:
        tasks = [executor.submit(httpx.get, url) for url in urls]

        for future in as_completed(tasks):
            try:
                resp = future.result()
                return resp.json()
            except Exception as e:
                exceptions.append(e)

    raise Exception("Download failed", exceptions)


import asyncio
from threading import Thread
from nonebot_desktop_wing.hindsight import BackgroundObject
from nonebot_desktop_wing.molecules import import_with_lock
from nonebot_desktop_wing.resources import load_module_data_raw


class _nb_cli:
    __singleton = None

    def __new__(cls):
        if cls.__singleton is None:
            cls.handlers = BackgroundObject(import_with_lock, "nb_cli.handlers", "*")
            cls.config = BackgroundObject(import_with_lock, "nb_cli.config", "*")
            cls.__singleton = object.__new__(cls)
        return cls.__singleton


nb_cli = _nb_cli

Thread(target=_nb_cli).start()


class _meta:
    __singleton = None

    def __new__(cls):
        if cls.__singleton is None:
            # load in __new__ to avoid lagging when getting `nb_cli.handlers`
            cls.drivers = BackgroundObject(asyncio.run, nb_cli().handlers.load_module_data("driver"))
            cls.adapters = BackgroundObject(asyncio.run, nb_cli().handlers.load_module_data("adapter"))
            cls.plugins = BackgroundObject(asyncio.run, nb_cli().handlers.load_module_data("plugin"))
            cls.raw_drivers = BackgroundObject(load_module_data_raw, "drivers")
            cls.raw_adapters = BackgroundObject(load_module_data_raw, "adapters")
            cls.raw_plugins = BackgroundObject(load_module_data_raw, "plugins")
            cls.__singleton = object.__new__(cls)
        return cls.__singleton


meta = _meta

Thread(target=_meta).start()